/*
 * $Id: common.h 146 2015-06-07 17:07:05Z ijuranek $
 *
 * Structures important for object detection.
 *
 * Roman Juranek <ijuranek@fit.vutbr.cz>
 * Faculty of Information Technology, BUT, Brno
 *
 */

#ifndef _COMMON_H_
#define _COMMON_H_

#include <vector>

// This is a general image representation structure compatible with MATLAB
// and OpenCV.
struct Image
{
    const float * data {0};
    int ndim {0};
    const int * dims {0};
    unsigned strides[3] {1,1,1};
    Image(){}
    Image(const float * d, int nd, const int * ds)
        :data(d), ndim(nd), dims(ds)
    {
        if (!data) return;
        strides[0] = 1;
        strides[1] = dims[0] * strides[0];
        strides[2] = dims[1] * strides[1];
    }
};

// Representation of bounding box
struct BB
{
    float x, y, width, height, scale;
    BB(float _x, float _y, float _w, float _h, float _s):
        x(_x), y(_y), width(_w), height(_h), scale(_s){};
};

// Structure with object detection result - list of detected boundig boxes,
// list of responses, and numbers of evaluated samples and features.
struct Result
{
    std::vector<BB> bbs;
    std::vector<float> hs;
    unsigned long long nf {0};
    unsigned long long ns {0};
};

#endif