#include "mex.h"
#include <vector>
#include <algorithm>
#include <ThreadPool.h>

// Accumulation kernel
void process(int i0, int i1, float * P, float * Ps, int U, const float * V, const float * init, const unsigned char * S, int M, std::vector<float*> & maps)
{
    for (int i = i0; i < i1; ++i)
    {
        float * Pi = P + U * i;
        std::copy(init, init+U, Pi);
        const unsigned char * Si = S + M * i;
        for (int j = 0; j < M; ++j)
        {
            const unsigned char n = Si[j];
            float * Mj = maps[j];
            float * Mjn = Mj + U * n;
            for (int k = 0; k < U; ++k)
            {
                Pi[k] += Mjn[k];
            }
        }
        int mi = std::distance(Pi, std::max_element(Pi, Pi+U)); // We dont give a ... about the actual value of maximum
        std::copy(V+3*mi, V+3*mi+3, Ps + 3*i);
    }
}

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[])
{	
	// Get number of samples
	const mxArray * mxS = prhs[0];
	const mxArray * mxD = prhs[1];
    
    // [N,K] = size(S);
    int M = mxGetM(mxS); // Number of stages (of regressor to eval)
    int N = mxGetN(mxS); // Number of samples
    const unsigned char * S = (const unsigned char*)mxGetData(mxS);

    // Data for pose model
    const mxArray * mxPoseModel = mxGetField(mxD, 0, "pose_model");
    const mxArray * mxMaps = mxGetField(mxPoseModel, 0, "map");
    const mxArray * mxInit = mxGetField(mxPoseModel, 0, "init");
    const float * init = (const float*)mxGetData(mxInit);
    const mxArray * mxV = mxGetField(mxPoseModel, 0, "V");
    const float * V = (const float*)mxGetData(mxV);
    int U = mxGetM(mxInit); // Size of regressors
    
    std::vector<float*> maps(mxGetNumberOfElements(mxMaps));
    for (int j = 0; j < maps.size(); ++j)
    {
        maps[j] = (float*)mxGetData(mxGetCell(mxMaps, j));
    }

    mxArray * mxP = mxCreateNumericMatrix(U, N, mxSINGLE_CLASS, mxREAL);
    float * P = (float*)mxGetData(mxP);
    mxArray * mxPs = mxCreateNumericMatrix(3, N, mxSINGLE_CLASS, mxREAL);
    float * Ps = (float*)mxGetData(mxPs);
    
    /*
    ThreadPool pool(4);
    int batch = 4;
    for (int i = 0; i < N; i+=batch)
    {
        pool.enqueue(process, i, std::min(i+batch, N), P, Ps, U, V, init, S, M, maps);
    }
    */
    
    process(0, N, P, Ps, U, V, init, S, M, maps);

    plhs[0] = mxPs;
    plhs[1] = mxP;
    
	return;
}