#include "wbt_detector.h"
#include <cmath>
#include <cassert>

#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif

using namespace std;

namespace WBTDetector
{

Detector * loadDetector(const char * filename)
{
    return 0;
}

void destroyDetector(Detector ** D)
{
    return;
}

bool initDetector(Detector & D)
{
    return true;
}


struct Response
{
    bool valid {false};
    int stgCount {0};
    int splitCount {0};
    float response {0};
    Signature signature;
    // Other stuff goes here
};

// Take image sample from (a,b,c) coordinates
float sample_nearest(const Image & im, float a, float b, float c)
{
    unsigned ind = round(a)*im.strides[0] + round(b)*im.strides[1] + round(c)*im.strides[2];
    return im.data[ind];
}

float npd(float a, float b) { return (a - b) / (a + b); }
float diff(float a, float b) { return a - b; }
float cmp(float a, float b) { return a; }

template <float(*SPLIT)(float,float)>
Response evalForest(const Image & im, const Detector & D, float x, float y)
{
    Response R;
    for (int tree = 0; tree < D.nStages; ++tree)
    {
        //int nTrees = D.stgEnd[stage] - D.stgStart[stage] + 1;
        //float stageResponse = 0.0f;
        //for (int tree = D.stgStart[stage]; tree <= D.stgEnd[stage]; ++tree)
        //{
            int * tp = D.tp + tree * D.nNodes;
            int * children = D.children + tree * D.nNodes;
            float * splits = D.splits + 7 * D.nNodes * tree;
            float * prob = D.prob + tree * D.nNodes;
            // Traverse the tree
            int node = 0;
            while (tp[node] != 0)
            {
                R.splitCount++;
                float * f = splits + 7 * node;
                float f1 = sample_nearest(im, x+f[0], y+f[1], f[4]);
                float f2 = sample_nearest(im, x+f[2], y+f[3], f[5]);
                int split = SPLIT(f1,f2) > f[6];
                node = children[node] + (split?0:1);
            } // tree traversal
            R.response += prob[node];
            if (tree >= (D.nTrees-SignatureSize))
            {
                R.signature[tree-(D.nTrees-SignatureSize)] = (unsigned char)(node);
                //#ifdef MATLAB_MEX_FILE
                //mexPrintf("%i, ", node);
                //#endif
            }
        //} // tree
        //stageResponse /= nTrees;
        //R.response += stageResponse;
        if (R.response < D.tau[tree])
        {
            R.stgCount = tree + 1;
            R.valid = false;
            return R;
        }
    } // stage/tree
    R.stgCount = D.nStages;
    R.valid = true;
    return R;
}


Result scanImage(const Image & im, const int x0, const int y0, const int x1, const int y1, const Detector & D, const float scale)
{
    Result res;
    Response (*evalFunc)(const Image&,const Detector&,float,float)=0;

    switch (D.type)
    {
    case 0:
        evalFunc = evalForest<cmp>;
        break;
    case 1:
        evalFunc = evalForest<diff>;
        break;
    case 2:
        evalFunc = evalForest<npd>;
        break;
    }

    assert(evalFunc != 0 && "Split evaluation function must be defined");
    
    for (int y = y0; y < y1; ++y)
        for (int x = x0; x < x1; ++x)
        {
            Response R = evalFunc(im, D, x, y);
            if (R.valid)
            {
                res.bbs.push_back(BB(x, y, D.w, D.h, scale));
                res.hs.push_back(R.response);
                res.ss.push_back(R.signature);
                /*
                #ifdef MATLAB_MEX_FILE
                Signature & a = res.ss.back();
                for (int i = 0; i < 64; ++i)
                    mexPrintf("%d, ", int(a[i]));
                mexPrintf("\n");
                #endif
                 */
            }
            res.nf += R.splitCount;
        }
    res.ns = (x1 - x0) * (y1 - y0); // Number of positions evaluated
    return res;
}

/*
Result scanImageMap(const Image & im, Image & map, const int x0, const int y0, const int x1, const int y1, const Detector & D)
{
   Result res;
   Response (*evalFunc)(const Image&,const Detector&,float,float)=0;

    switch (D.type)
    {
    case 0:
        evalFunc = evalForest<cmp>;
        break;
    case 1:
        evalFunc = evalForest<diff>;
        break;
    case 2:
        evalFunc = evalForest<npd>;
        break;
    }

    assert(evalFunc != 0 && "Split evaluation function must be defined");

    int map_px;
    
    for (int y = y0; y < y1; ++y)
    {
        for (int x = x0; x < x1; ++x)
        {
            Response R = evalFunc(im, D, x, y);
            float val = -INFINITY;

            if (R.valid)
            {
                res.bbs.push_back(BB(x, y, D.w, D.h, scale));
                res.hs.push_back(R.response);
                res.ss.push_back(R.signature);
                val = res.response;
            }

            map[y*map.strides[0] + x*map.strides[1]] = val;

            res.nf += R.splitCount;
        }
    }
    res.ns = (x1 - x0) * (y1 - y0); // Number of positions evaluated
    return res;
}
*/

} // namespace
