#ifndef _WBT_DETECTOR_H_
#define _WBT_DETECTOR_H_

#include "common.h"
#include <vector>
#include <array>

namespace WBTDetector
{

struct Detector
{
    int * tp;       // Node type
    float * splits;  // Split parameters
    int * children; // Indices of left child node
    float * prob;   // Probability matrix
    float * tau;
    // int * stgStart;
    // int * stgEnd;
    int nStages;
    int nTrees;
    int nNodes;
    int w;
    int h;
    int type;
};

const int SignatureSize = 128; // Maximal length of sample signature

typedef std::array<unsigned char,SignatureSize> Signature;

struct Result
{
    std::vector<BB> bbs;
    std::vector<float> hs;
    std::vector<Signature> ss;
    unsigned long long nf {0};
    unsigned long long ns {0};
};

Detector * loadDetector(const char * filename);

void destroyDetector(Detector ** D);

bool initDetector(Detector & D);

// Scan an area defined by [x0,y0,x1,y1] in image im by detector D;
Result scanImage(const Image & im, const int x0, const int y0, const int x1, const int y1, const Detector & D, const float scale);

} // namespace

#endif
