function [H,ind,I] = eval_detector(D, X)
% Evaluate detector D on samples S.
%
% Function H = EVAL_DETECTOR(D,S) returns vector of responses of detector D
% on samples S. The samples must be stored in structure array with field
% chns containing matrix with extracted feature channels. Although it is
% not strictly checked, all samples must have same size.

nsamples = size(X,4);
nstages = numel(D.tree);
H = zeros(nsamples, 1);
ind = int32(1:nsamples);
I = cell(1,nstages);
if isempty(D) || isempty(D.tree), return; end;
% Eval cascade
for s = 1:nstages
    %disp(s);
    [p,J] = random_forest_predict(D,X,ind,s);
    I(s) = J(s);
    H(ind) = H(ind) + p;
    rejected = H(ind) < D.tau(s);
    ind(rejected) = [];
    if isempty(ind), break; end;
end