function [H] = eval_detector_ext(D, X)
% Evaluate detector D on samples S.
%
% Function H = EVAL_DETECTOR(D,S) returns vector of responses of detector D
% on samples S. The samples must be stored in structure array with field
% chns containing matrix with extracted feature channels. Although it is
% not strictly checked, all samples must have same size.
%
% D.tree        Cell array with trees
% D.stg         Numbers of trees in stages
% D.tau         Threshold for every stage
%
nsamples = size(X,4);
nstages = numel(D.tree);
H = nan(nsamples, nstages+1);
H(:,1) = 0;
ind = int32(1:nsamples);
I = cell(1,nstages);
if isempty(D) || isempty(D.tree), return; end;
% Eval cascade
for s = 1:nstages
    %disp(s);
    [p,J] = random_forest_predict(D,X,ind,s);
    I(s) = J(s);
    H(ind,s+1) = H(ind,s) + p;
    rejected = H(ind,s+1) < D.tau(s);
    ind(rejected) = [];
    if isempty(ind), break; end;
end