function [S,H,R,L] = sample_from_detections(I, N, detector, R0, gt)
% Use detector to generate samples from image

% S - sample matrices
% H = sample responses
% R - positions of samples
% L - total available samples

[S,H,R] = deal([]);
L = 0;

% Detect bb candidates
%[bbs_scale,~,~,bbs,imgs] = detect_image(I,detector,false);

pyr = chnsPyramid(I, detector.opts.pPyramid);
bbs = mxScanImages(pyr.data, detector);
shrink = detector.opts.pPyramid.pChns.shrink;
pad = detector.opts.pPyramid.pad ./ shrink;
s = shrink./pyr.scaleshw(bbs(6,:),:)';
bbs_scale = bbs;
bbs_scale(1:2,:) = bsxfun(@minus, bbs_scale(1:2,:), pad');
bbs_scale(1:2,:) = bsxfun(@times, bbs_scale(1:2,:), s) + 1;
bbs_scale(3:4,:) = bsxfun(@times, bbs_scale(3:4,:), s);
bbs(1:2,:) = bbs(1:2,:)+1;

bbs_scale = bbs_scale';
bbs = bbs';

if isempty(bbs), return; end;

%{
bbs_scale_max = bbNms(bbs_scale,'type','max');
oas = bbGt('compOas',bbs_scale, bbs_scale_max);
[~,ids] = max(oas);
bbs = bbs(ids,:);
bbs_scale = bbs_scale(ids,:);
%}

H = bbs(:,5);
R = bbs(:,[1 2 6]);
L = size(bbs, 1);

%%% Reduce number of proposals

if size(bbs_scale,1) > 1000
    i = sort(randsample(size(bbs_scale,1), 1000));
    bbs_scale = bbs_scale(i,:);
    H = H(i,:);
    R = R(i,:);
end

%%% Filter according to image ground truth

keep_gt = filter_bbs_gt(bbs_scale, gt, 0.3);
keep_loaded = filter_bbs(R, R0);
keep = keep_gt & keep_loaded;

H = H(keep,:);
R = R(keep,:);
bbs_scale = bbs_scale(keep,:); % just for completenes

%%% Select N best windows

if length(H) > N
    %[~,ord] = sort(H,'descend');
    %ids1 = ord(1:min(length(ord),N));
    keep = randsample(length(H), N);
    R = R(keep,:);
    H = H(keep);
    bbs_scale = bbs_scale(keep,:);
end
%{

figure(16);
imshow(I);
bbApply('draw', bbs_scale(:,1:4),'g',1);
drawnow;
%}

% Finally crop samples
S = crop_bbs(pyr.data, R, detector.dims);

L = L - size(S,4);