function [S,V] = sample_from_gt(N, opts)
% Crop annotated objects and extract features

if size(opts.fs,1) < 2
    error('GT must be specified');
end

hasvpt = size(opts.fs,1) > 2;

nImg = size(opts.fs,2);
if isfield(opts,'random') && opts.random, ord = randperm(nImg);
else ord = 1:nImg; end;

[S,V] = deal(cell(1,nImg));

loaded = 0;
tic = ticStatus('Loading objects...', 0.5, 5);
for i = 1:nImg
    k = ord(i);
    
    % Load image and ground truth
    [~,gt,keep]=bbGt('bbLoad',opts.fs{2,k},opts.pLoad);
    ign = gt(:,5)==1; gt(ign,:) = [];% Discard ignored entries
    if isempty(gt), continue; end;
    im = feval(opts.imreadf,opts.fs{1,k},opts.imreadp{:});
    if ismatrix(im), im=repmat(im,[1,1,3]); end;
    
    max_samples = ceil(size(gt,1) * opts.nPerPos);
    
    [S{i},n] = sample_from_gt1(im, gt, opts);
    
    if hasvpt
        vpt = csvread(opts.fs{3,k});
        vpt(~keep,:) = [];
        vpt(ign,:) = [];
        %vpt = vpt(:,1:3);
        %V{i} = reshape(repmat(reshape(vpt',3,1,[]),1,n),3,[])';
        V{i} = reprows(vpt, n);
    end;
    %%{
    nums = size(S{i},4);
    if  nums > max_samples,
        q = randsample(nums, max_samples);
        S{i} = S{i}(:,:,:,q);
        if hasvpt
            V{i} = V{i}(q,:);
        end
    end
    %}
    
    loaded = loaded + size(S{i},4);
    if loaded >= N, break; end;
    tocStatus(tic, loaded/N);
end
tocStatus(tic, 1);

S = cat(4, S{:});

if hasvpt, V = cat(1, V{:});
else V = []; end;

fprintf('Sampled %i samples from %i images\n', loaded, i);

end

function [S,n] = sample_from_gt1(im, bbs, opts)

shrink = opts.pPyramid.pChns.shrink;

% Grow bbs and crop the windows
% taken from Piotr's Toolbox
modelDsBig=max(8*shrink,opts.modelDsPad)+max(2,ceil(64/shrink))*shrink;
r=opts.modelDs(2)/opts.modelDs(1); assert(all(abs(bbs(:,3)./bbs(:,4)-r)<1e-5));
r=modelDsBig./opts.modelDs;
bbs=bbApply('resize',bbs,r(1),r(2));
Is=bbApply('crop',im,double(bbs),'replicate',modelDsBig([2 1]));
Is=cat(4,Is{:}); % MxNxCxK matrix
n = 1;
if isstruct(opts.pJitter)
    [~,~,~,k0] = size(Is);
    opts.pJitter.hasChn = 1;
    Is = jitterImage(Is, opts.pJitter);
    [~,~,~,k] = size(Is);
    n = k / k0;
end

% Is is MxNxCxKxR matrix

ds=size(Is);
cr=rem(ds(1:2),shrink);
s=floor(cr/2)+1;
e=ceil(cr/2);
Is=Is(s(1):end-e(1),s(2):end-e(2),:,:,:);
ds=size(Is);
if(any(ds(1:2)<opts.modelDsPad)), error('Windows too small.'); end;

S = compute_channels(Is, opts.pPyramid, opts.modelDsPad);

end

function X = compute_channels(Is, pPyramid, modelDsPad)

if isempty(Is), X = []; return; end
pChns = pPyramid.pChns;
smooth = pPyramid.smooth;

ds=size(Is); ds(1:3)=1;
Is=squeeze(mat2cell2(Is,ds));
n=numel(Is);
modelDsPad = modelDsPad/pPyramid.pChns.shrink;

X = cell(1,n);

for i=1:n
    C=chnsCompute(Is{i},pChns);
    C = convTri(cat(3,C.data{:}),smooth);
    ds=size(C); cr=ds(1:2)-modelDsPad; s=floor(cr/2)+1; e=ceil(cr/2);
    X{i} = single(C(s(1):end-e(1),s(2):end-e(2),:));
end;

X = cat(4, X{:});

end

