function [S,R] = sample_random_wins(I, N, pPyramid, dim, R0, gt)
% Crop random windows from image
%
% Returns:
%   S       MxNxCxK matrix with samples
%   R       Kx3 matrix with identity of taken samples [x,y,scale_id]

S = []; R = [];

[h,w,~] = size(I);
shrink = pPyramid.pChns.shrink;
if any([h,w]/shrink <= dim); return; end;

J = chnsPyramid(I, pPyramid);
[imgs,scales] = deal(J.data, J.scales);
%nchns = size(imgs{1},3);

s = scales.^2; s = s/sum(s);
m = (4*N * s) > 1;
N1 = ceil((4*N * s) .* m);

%nPerScale = ceil((4*N)./numel(scales));
bbs = cell(1,numel(imgs));

dim = double(dim);
% generate bbs
for i = 1:numel(imgs)
    im = imgs{i}; [h,w,~] = size(im);
    if any([h,w] <= dim), break; end;
    nPerScale = N1(i);
    % generate nPerScale fixed size (dim) bbs
    bb = zeros(nPerScale,5);
    bb(:,1) = ceil((w-dim(2)) * rand(nPerScale,1));
    bb(:,2) = ceil((h-dim(1)) * rand(nPerScale,1));
    bb(:,3) = dim(2);
    bb(:,4) = dim(1);
    bb(:,5) = i;
    bbs{i} = bb;
end
bbs = vertcat(bbs{:}); % [x,y,d1,d2,scale_id]

if isempty(bbs), bbs = zeros(0,5); end;

R = bbs(:,[1,2,5]);
keep = filter_bbs(R, R0);
R = R(keep,:);

if size(R,1) > N
    keep = randsample(size(R,1), N);
    R = R(keep,:);
end

S = crop_bbs(imgs, R, dim);