function [S,H,sample_info] = sample_windows(N, opts, sample_info)
% Crop samples from images using detector
% N             Number of samples to load
% opts
%  .fs          1xK or 2xK or 3xK cell array with file names
%  .imreadp
%  .imreadf
%  .pPyramid
%  .dim
%  .pLoad
%  .nPerImage
%
%  .detector

hasGt = size(opts.fs,1) >= 2; % Use for filtering
nImg = size(opts.fs,2);

S = []; H = [];

for i = 1:nImg,
    if sample_info(i).left ~= 0, sample_info(i).left = nan; end;
end;

max_trials = 4;
for trial = 1:max_trials
    ord = randperm(nImg);
    [tmp_S,tmp_H] = deal(cell(1,nImg));
    trial_imgs = 0; trial_loaded = 0;
    need_samples = N-size(S,4) + 1;
    tic = ticStatus(sprintf('Bootstrapping trial %i/%i (need %i windows)...', trial, max_trials, need_samples), 1, 0);
    for i = 1:nImg
        k = ord(i);
        % decide to skip image based on sample_info
        if ~isnan(sample_info(k).left) && sample_info(k).left == 0, continue; end;
        im = feval(opts.imreadf,opts.fs{1,k},opts.imreadp{:});
        gt=[]; if(hasGt), [~,gt]=bbGt('bbLoad',opts.fs{2,k},opts.pLoad); end;
        trial_imgs = trial_imgs+1;
        
        if ~isstruct(opts.detector)
            [samples_k,R] = sample_random_wins(im, opts.nPerImage, opts.pPyramid, opts.dim, sample_info(k).R, gt);
            nloaded = size(samples_k,4);
            responses_k = zeros(nloaded,1);
            nleft = nan;
        else
            [samples_k,responses_k,R,nleft] = sample_from_detections(im, opts.nPerImage, opts.detector, sample_info(k).R, gt);
            nloaded = size(samples_k,4);
        end
        
        sample_info(k).left = nleft;
        
        if isempty(samples_k), continue; end; % Nothing loaded, skip
        
        sample_info(k).loaded = sample_info(k).loaded + nloaded;
        sample_info(k).R = cat(1, sample_info(k).R, R);
        
        
        tmp_S{k} = samples_k;
        tmp_H{k} = responses_k;
        trial_loaded = trial_loaded + nloaded;
        if trial_loaded>=need_samples, break; end;
        tocStatus(tic, trial_loaded/need_samples);
    end
    tocStatus(tic, 1);
    
    S = cat(4, S, tmp_S{:});
    H = cat(1, H, tmp_H{:});
    
    fprintf('Sampled %i windows from %i images\n', trial_loaded, trial_imgs);
    
    if size(S,4)>=N || trial_loaded==0, break; end;
    
end
