function [F,splits,err] = pixel_split(N, X, ~, ~, ind, ~, B, dist, tp)
% The function generates N pixel comparison features

err = [];
thr_per_split = 1;
N = ceil(N / thr_per_split);
[u,v,w,~] = size(X);
F = px_cmp(N, [u,v,w], B, dist);
F = repmat(F, thr_per_split, 1);
[~,M] = pixel_split_eval(X,ind,F,tp);

m0 = min(M,[],1)'; m1 = max(M,[],1)';
thr = m0 + rand(size(F,1),1) .* (m1-m0);

%[thr,err] = optimize_splits(single(M), uint8(Y(ind)), single(W(ind)));

if ~isrow(thr), thr = thr'; end;

F(:,7) = thr';
splits = bsxfun(@gt, M, thr);

end


function F = px_cmp(n, dims, bin_matrix, dist)
% Generate random pixel comparison features
%
%   F = PX_CMP_FEATURES(N, DIMS, B)
%   F = PX_CMP_FEATURES(N, DIMS, B, DIST)
%
% See also: PX_CMP_EVAL, RANDOM_PLANES

if ~isvector(dims) || numel(dims) < 2 || numel(dims) > 3
    error('Dimensions must be specified in vector of two or 3 elements');
end

if length(dims) == 2
    ch = 1;
else
    ch = dims(3);
end

if ~ismatrix(bin_matrix) || size(bin_matrix,1) ~= size(bin_matrix,2) || size(bin_matrix,1) ~= ch
    error('B must be square matrix of size C x C where C = dims(3) or C = 1');
end

if nargin < 4
    dist = 'uniform';
end

bins = find(isfinite(bin_matrix));
[i,j] = ind2sub(size(bin_matrix), bins);
T = bin_matrix(bins);
F_bin = randi(numel(T), n, 1);
F_t = zeros(n, 1);

max_dims = repmat(dims(1:2), 1, 2);

switch dist
    case 'uniform'
        F_px = ceil(bsxfun(@times, rand(n, 4), max_dims));
    case 'gaussian'
        F_px = ceil(...
            bsxfun(@plus,...
            bsxfun(@times, randn(n, 4), max_dims/5),...
            max_dims/2));
        F_px = clamp(F_px, max_dims);
    otherwise
        error('Unknown distribution type');
end

F = [F_px, i(F_bin), j(F_bin), F_t];

    function X = clamp(X, r)
        for c = 1:size(X, 2)
            X(:,c) = max(min(X(:,c), r(c)), 1);
        end
    end
end % px_cmp