%% DEMO
%
% This is experimental implementation of paper
% 
%   Juranek et al.: Real-time Pose Estimation Piggybacked on Object Detetion, ICCV, 2015

resultdir = '.';

% COD20K dataset - http://www.fit.vutbr.cz/research/groups/graph/PoseEstimation
data = '/mnt/matylda1/juranek/Datasets/COD20K_RELEASE/';
img = fullfile(data, 'frames', 'train');
gt = fullfile(data, 'annotations', 'train');
vpt = fullfile(data, 'viewpoints', 'train');

testimg = fullfile(data, 'frames', 'test');

% VOC dataset
bgdata = '/home/ijuranek/Data/VOC';
bgimg = fullfile(bgdata, 'JPEGImages');
bggt = fullfile(bgdata, 'Annotations');

% Parameters for pose model
load sphere;

%% Setup parameters

opts = wbTrainDetector;

% Set up channels
pPyramid = chnsPyramid;
pPyramid.pChns.shrink = 4;
pPyramid.pChns.pColor.colorSpace = 'gray';
pPyramid.lambdas = [];
pPyramid.nApprox = 0;

% Sample sizes
opts.pPyramid = pPyramid;
opts.modelDs = [52,52]; opts.modelDsPad = [64,64];
opts.imreadf = @imread; opts.imreadp = {};
opts.pNms = {'type','maxg','thr',0};

% Source of negative samples
opts.negImgDir = bgimg;
opts.negGtDir = bggt;
opts.pLoadNeg = {'lbls',{'Car','Vehicle'},'format',1}; % ignore specified lbls
opts.nPerNeg = 25;
opts.nNeg = 2000;

% Source of positive samples
opts.posGtDir = gt;
opts.posImgDir = img;
opts.posVptDir = vpt;
opts.pLoadPos = {'lbls',{'car','car_from'},'squarify',{4,1},'wRng',[30 inf]};
opts.nPos = 2000;
opts.subsample = inf;
opts.pJitter = [];

% Weak learner
opts.pTree.verbose = 1;
opts.pTree.splits = 100;
opts.pTree.max_depth = 2;
opts.pTree.min_samples = 25;
opts.pTree.split_gen = {'pixel_split',{eye(8),'uniform','diff'}};
opts.pTree.split_eval = {'pixel_split_eval',{'diff'}};

% Cascade settings
opts.nWeak = 136;
opts.show_stats = 0;
thr = -inf(1,128); thr([4 6 8 12 16 32 64]) = -1;
opts.thr = thr;
opts.logfile = 'detector.log';
opts.save.every = 32;

% Pose model
opts.pose_model = struct('V',S,'start',9,'error','regression','classes',false,'alpha',0.3,'depth',5);


%%

detector = wbTrainDetector(opts);

%%

%load detector.mat;
detector = D;
detector.opts.pNms = {'type','max','thr',10,'overlap',0.5};
detector.opts.pPyramid.nOctUp = 1;
detector.opts.pPyramid.nPerOct = 12;
detector.opts.AccOvr = 0.5;

imgs = bbGt('getFiles',{testimg});
detector = repairPoseModel(detector);
d = preprocess_detector(detector);
for i = randperm(numel(imgs))
    im = imread(imgs{i});
    
    bb = wbDetect(im,d,1); % Detection and map accumulation
    maps = bb(:,6:end); [~,k] = max(maps,[],2); ps = detector.pose_model.V(:,k)'; % Get real poses
    
    %%% This is just result display
    imshow(im);
    bbApply('draw',bb,'g',1,'-');
    %%{
    c = bbApply( 'getCenter', bb );
    [sz(1),sz(2),~] = size(im);
    for j = 1:size(bb,1)
         draw_pose(c(j,:), c(j,:), ps(j,:), 3, 40, 1500, sz/2); % 1500 is focal length in pixels - it does not correspond to actual focal of the images so the crosses may seem little bit distorted!
    end
    hold off;
    %}
    pause;
end
