function [bbs_out,ns,nf] = wbDetect(I, D, pose)
% Detect objects in image using WaldBoost Trees detector.
%
%   [BBS,S,F] = WBT_DETECT(I, D)
%

% INPUT CHECKING
[bbs_out,ns,nf] = deal(zeros(0,5), 0, 0); % Empty output
[h,w,~] = size(I);
if any([h,w] < D.opts.modelDsPad), return; end;

if nargin < 3, pose = true; end;

%%% IMAGE PYRAMID
pyr = chnsPyramid(I, D.opts.pPyramid);

%%% DETECTION
[bbs,~,~,sgns] = mxScanImages(pyr.data, D); % Actual detection
%disp(nf/ns);
bbs = double(bbs);
shrink = D.opts.pPyramid.pChns.shrink;
pad = D.opts.pPyramid.pad ./ shrink;
s = shrink./pyr.scaleshw(bbs(6,:),:)';
bbs(1:2,:) = bsxfun(@minus, bbs(1:2,:), pad');
bbs(1:2,:) = bsxfun(@times, bbs(1:2,:), s) + 1;
bbs(3:4,:) = bsxfun(@times, bbs(3:4,:), s);
bbs = bbs(1:5,:)';
bbs_out = bbNms(bbs, D.opts.pNms);

%%% POSE ESTIMATION
if isfield(D, 'pose_model') && ~isempty(bbs_out) && pose
    % Get pose maps
    oa = bbGt('compOas', bbs_out, bbs);
    valid = max(oa,[],1) > D.opts.AccOvr;
    oa = oa(:,valid); sgns = sgns(:,valid);
    [~,M] = mxGetPoses(sgns,D);
    m_out = zeros(size(M,1), size(bbs_out,1), 'single');
    for i = 1:size(m_out,2), m_out(:,i) = sum(M(:,oa(i,:)>0), 2); end
    bbs_out = horzcat(bbs_out, m_out');
end