function [gt,dt] = test_detector(detect, pDetect, imgNms, gtNms, pLoad)
%
% [ROC,PRC] = TEST_DETECTOR(detect, pDetect, imgNms, gtNms, pLoad);
%
% detect    Detection function, for example @acfDetect
% pDetect   Arguments of the function
% imgNms    Image filenames
% gtNms     Ground Truth filenames
% pLoad     Parameters for loading bounding boxes from gt (see bbGt>bbLoad)
%
% The detection function *MUST* have following format:
%   [bbs,...] = detect(I,pDetect);
% In other cases, anonymous function can be used.
%
% Where bbs is a matrix with detections stored in rows (x,y,w,h,wt), and I
% is an image (grayscale or RGB). The detector must be passed in pDetect
% arguments along with other additional inputs.
%
% Examples:
%   [ROC,PRC] = TEST_DETECTOR(@acfDetect,{D},imgNms, gtNms, {'lbls',{'face'},'hRng',[40,inf],'wRng',[40,inf]});
%
% See also: acfDetect, waldboost_detect_lbp, detect_image, bbGt>bbLoad

[gt,dt] = deal(cell(1,numel(imgNms)));
for i = 1:numel(imgNms)
    fprintf('%i/%i\n', i, numel(imgNms));
    I = imread(imgNms{i});
    [~,gt{i}] = bbGt('bbLoad', gtNms{i}, pLoad);
    t0 = tic;
    dt{i} = feval(detect, I, pDetect{:});
    t = toc(t0);
    t
end
%{
overlap = 0.1:0.1:0.9;
[gt1,dt1] = deal(cell(1,numel(overlap)));
for i = 1:numel(overlap)
    [gt1{i},dt1{i}] = bbGt('evalRes',gt,dtx,overlap(i));
end
%}
%{
[fppi,tp,score1] = bbGt('compRoc',gt1,dt1,1,[]);
[recall,precision,score2] = bbGt('compRoc',gt1,dt1,0,[]);
ROC = [score1, fppi, tp];
PRC = [score2 precision, recall];
%}